/*****************************************************************************/
/*									     */
/*									     */
/*	Xpack version 1.00 -- module commands.c				     */
/*									     */
/*	Most of the entries for commands assignable to keys		     */
/*	Written by Michael Bischoff (mbi@mo.math.nat.tu-bs.de)		     */
/*	November-1994							     */
/*	see COPYRIGHT.xpack for Copyright details			     */
/*									     */
/*									     */
/*****************************************************************************/
#ifndef _POSIX_SOURCE
#define _POSIX_SOURCE
#endif
#include <sys/types.h>
#include <unistd.h>
#include "xpack.h"
#include "version.h"

void cmd_Redraw(void) {
    doPaint(0, 0, P.t[0].width+1, P.t[0].height+1);
}

void cmd_ShowArea(void) {
    show_message("The area is %d*%d = %d units (min %d free)",
		 P.t[0].width, P.t[0].height,
		 P.t[0].width * P.t[0].height,
		 P.t[0].width * P.t[0].height - P.maxscore);
}

void cmd_LevelInfo(void) {
    char s[256], *t;
    int i;
    sprintf(s, "Unused:");
    for (i = 1; i <= P.totaltiles; ++i)
	if (P.position[i].x < 0) {
	    t = s + strlen(s);
	    sprintf(t, " %s", P.t[P.position[i].tilenr].name);
	}
    show_message(s);
}

void rq_LeavePack(void) {
    request_confirm(cmd_LeavePack, "Quit game?");
}

/* unused, since this can be undone */
void rq_RestartGame(void) {
    request_confirm(cmd_RestartGame, "Restart game?");
}

void cmd_RestartGame(void) {
    clear_position();
    cmd_Redraw();
}

void cmd_Heuristik1(void) {
    clear_position();
    make_simple_solution();
    cmd_Redraw();
}

void cmd_PlaceLargest(void) {
    int i;
    if (active) {
	show_message("There is an active tile");
	return;
    }
    for (i = 1; i <= P.totaltiles; ++i) {
	if (P.position[i].x < 0) {
	    int x, y;
	    activate_free(i);
	    for (y = 0; y <= P.t[0].height-a.h; ++y)
		for (x = 0; x <= P.t[0].width-a.w; ++x) {
		    if (place_and_redraw(x, y)) {
			show_message("Tile %s placed at %d,%d",
				     P.t[a.tilenr].name, x, y);
			cmd_Redraw();
			return;
		    }
		}
	    discard_active();
	}
    }
    show_message("No further tile could be placed");
}

static const char *filename(const char *ext) {
    static char str[256], *s;
    sprintf(str, "%s.%s", P.description, ext);
    for (s = str; *s; ++s) {
	if (*s == ' ')
	    *s = '_';
	else if (*s == '*')
	    *s = 'x';
    }
    return str;
}

void cmd_SaveGame(void) {
    FILE *fp;
    int i, nfree;
    const char *s;
    s = filename("sav");
    nfree = compute_score();
    if (!(fp = fopen(s, "w"))) {
	show_message("Could not write to %s", s);
	return;
    }
    fprintf(fp, "# save file for problem %s\n# score = %d, %d squares free\n",
	    P.description, game.score, nfree);
    for (i = 1; i <= P.totaltiles; ++i)
	fprintf(fp, "%d %d %d\n", P.position[i].type,
		P.position[i].x, P.position[i].y);
    fclose(fp);
    show_message("Position saved in %s", s);
}

void cmd_LoadGame(void) {
    FILE *fp;
    int i = 0;
    char line[256];
    const char *s;
    struct position *pp = P.position;

    s = filename("sav");
    if (!(fp = fopen(s, "r"))) {
	show_message("The file %s could not be opened", s);
	return;
    }
    clear_position();
    while (i < P.totaltiles) {
	if (!fgets(line, sizeof(line), fp))
	    break;
	if (*line == '#')
	    continue;
	++i; ++pp;
	sscanf(line, "%d %d %d", &pp->type,
	       &pp->x, &pp->y);
	if (pp->x >= 0) {
	    activate_free(i);
	    set_type(pp->type);
	    if (!place_active(pp->x, pp->y))
		fatal("Load error ");
	}
    }
    fclose(fp);
    if (i == P.totaltiles)
	show_message("Successfully loaded");
    else
	show_message("Short file!");
    cmd_Redraw();
}

void cmd_write_LaTeX(void) {
    FILE *fp;
    const char *s;
    int i, x, y;
    s = filename("tex");
    if (!(fp = fopen(s, "w"))) {
	show_message("Could not open LaTeX file %s", s);
	return;
    }
    fprintf(fp, "\\unitlength=1mm\n\\linethickness{.1mm}\n\\begin{picture}(%d,%d)\n",
	    P.t[0].width, P.t[0].height);
    fprintf(fp, "  \\put(0,0){\\framebox(%d,%d){}}\n",
	    P.t[0].width, P.t[0].height);
    for (i = 1; i <= P.totaltiles; ++i)
	if (P.position[i].x >= 0) {
	    int w, h;
	    if (P.position[i].type & 1) {
		h = P.t[i].width; w =  P.t[i].height;
	    } else {
		w = P.t[i].width; h =  P.t[i].height;
	    }
	    fprintf(fp, "  \\put(%d.05,%d.05){\\framebox(%d.9,%d.9){%s}}\n",
		    P.position[i].x, P.position[i].y, w-1, h-1,
		    w > 3 && h > 3 ? P.t[P.position[i].tilenr].name : "");
	}
    for (x = 0; x < P.t[0].width; ++x)
	for (y = 0; y < P.t[0].height; ++y)
	    if (P.board[y+1][x+1] == 0)
		fprintf(fp, "  \\put(%d.05,%d.05){\\framebox(.9,.9){$\\cdot$}}\n",
		    x, y);
    fprintf(fp, "\\end{picture}\n");
    fclose(fp);
    show_message("Picture written to %s", s);
}


void cmd_write_PostScript(void) {
    FILE *fp;
    const char *s;
    int i, x, y;
    s = filename("eps");
    if (!(fp = fopen(s, "w"))) {
	show_message("Could not open PostScript file %s", s);
	return;
    }
#define C	3	/* scale */
    fprintf(fp, "%%!PS-Adobe-2.0 EPSF-2.0\n"
	    "%%%%Title: %s.eps\n"
	    "%%%%BoundingBox: 0 0 %d %d\n"
	    "/Times-Roman findfont 3 scalefont setfont\n"
	    "%d %d scale\n"
	    ".1 setlinewidth\n"
	    "1 setlinecap\n"	/* round caps */
	    "2 setlinejoin\n"
	    "0 setgray\n"
	    "/cshow { dup stringwidth pop 2 div 0 exch sub 0 rmoveto show } def\n"
	    "[1 0] 0 setdash\n", s,
	    C*P.t[0].width, C*P.t[0].height, C, C);
    fprintf(fp, "newpath\n"
	    "0 0 moveto %d 0 lineto %d %d lineto 0 %d lineto closepath\n"
	    "gsave 0.7 setgray fill\n",
	    P.t[0].width, P.t[0].width, P.t[0].height, P.t[0].height);
    fprintf(fp, "1 setgray\n");
    for (x = 1; x < P.t[0].width; ++x)
	fprintf(fp, "newpath %d 0 moveto %d %d lineto stroke\n",
		x, x, P.t[0].height);
    for (y = 1; y < P.t[0].height; ++y)
	fprintf(fp, "newpath 0 %d moveto %d %d lineto stroke\n",
		y, P.t[0].width, y);
    fprintf(fp, "grestore gsave .2 setlinewidth stroke grestore\n");	/* draw border */

    for (i = 1; i <= P.totaltiles; ++i)
	if (P.position[i].x >= 0) {
	    int w, h;
	    if (P.position[i].type & 1) {
		h = P.t[i].width; w =  P.t[i].height;
	    } else {
		w = P.t[i].width; h =  P.t[i].height;
	    }
	    fprintf(fp, "newpath %d %d moveto %d %d lineto %d %d lineto "
		    "%d %d lineto closepath\ngsave 1 setgray fill grestore stroke\n",
		    P.position[i].x, P.position[i].y,
		    P.position[i].x+w, P.position[i].y,
		    P.position[i].x+w, P.position[i].y+h,
		    P.position[i].x, P.position[i].y+h);
	    if (w > 2 && h > 2)
		fprintf(fp, "%f %f moveto (%s) cshow\n",
		    P.position[i].x+((double)w *.5),
		    P.position[i].y+((double)h *.5)-1.2,
		    P.t[P.position[i].tilenr].name);
	}
    fprintf(fp, "showpage\n");
    fclose(fp);
    show_message("Picture written to %s", s);
}

void cmd_ShowVersion(void) {
    show_message("This is xpack version %s", VERSION);
}

void cmd_ShowScore(void) {
    int nfree = compute_score();
    show_message("Score is %d of %d. There are %d units free.",
		 game.score, P.maxscore, nfree);
}
